"""
A python module for reading and changing status of verisure devices through
mypages.
"""

__all__ = [
    'Error',
    'LoggedOutError',
    'LoginError',
    'MaintenanceError',
    'MyPages',
    'ResponseError',
    'TemporarilyUnavailableError',
    ]

from .mypages import MyPages
from .session import (
    Error,
    LoggedOutError,
    LoginError,
    MaintenanceError,
    ResponseError,
    TemporarilyUnavailableError,
    )


ALARM_ARMED_HOME = 'ARMED_HOME'
ALARM_ARMED_AWAY = 'ARMED_AWAY'
ALARM_DISARMED = 'DISARMED'
LOCK_LOCKED = 'LOCKED'
LOCK_UNLOCKED = 'UNLOCKED'
SMARTPLUG_ON = 'on'
SMARTPLUG_OFF = 'off'



"""
Vacationmode device
"""

from .overview import Overview

OVERVIEW_URL = '/overview/vacationmode'


class Vacationmode(object):
    """ Vacationmode device
        Args:
            session (verisure.session): Current session
    """

    def __init__(self, session):
        self._session = session

    def get(self):
        """ Get device overview """
        status = self._session.get(OVERVIEW_URL)
        return [Overview('vacationmode', status)]



""" submodule containing supported devices """

__all__ = [
    'Alarm',
    'Climate',
    'Ethernet',
    'Heatpump',
    'Lock',
    'Mousedetection',
    'Smartcam',
    'Smartplug',
    'Vacationmode',
    ]

from .alarm import Alarm
from .climate import Climate
from .ethernet import Ethernet
from .heatpump import Heatpump
from .lock import Lock
from .mousedetection import Mousedetection
from .smartcam import Smartcam
from .smartplug import Smartplug
from .vacationmode import Vacationmode



"""
API to communicate with Verisure mypages
"""

import verisure.devices as devices

from verisure.session import Session


class MyPages(object):
    """ Interface to verisure MyPages

    Args:
        username (str): Username used to log in to mypages
        password (str): Password used to log in to mypages
    """

    def __init__(self, username, password):
        self._session = Session(username, password)

        self.alarm = devices.Alarm(self._session)
        self.climate = devices.Climate(self._session)
        self.ethernet = devices.Ethernet(self._session)
        self.heatpump = devices.Heatpump(self._session)
        self.lock = devices.Lock(self._session)
        self.mousedetection = devices.Mousedetection(self._session)
        self.smartcam = devices.Smartcam(self._session)
        self.smartplug = devices.Smartplug(self._session)
        self.vacationmode = devices.Vacationmode(self._session)

    def __enter__(self):
        """ Enter context manager, open session """
        self.login()
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        """ Exit context manager, close session """
        self.logout()

    def login(self):
        """ Login to mypages

        Login before calling any read or write commands

        """
        self._session.login()

    def get_overviews(self):
        """ Get overviews from all devices """
        overviews = []
        overviews.extend(self.alarm.get())
        overviews.extend(self.climate.get())
        overviews.extend(self.ethernet.get())
        overviews.extend(self.heatpump.get())
        overviews.extend(self.lock.get())
        overviews.extend(self.mousedetection.get())
        overviews.extend(self.smartcam.get())
        overviews.extend(self.smartplug.get())
        overviews.extend(self.vacationmode.get())
        return overviews

    def logout(self):
        """ Ends session

        note:
            Ends the session, will not call the logout command on mypages

        """
        self._session.logout()



"""
Ethernet device
"""

from .overview import Overview

OVERVIEW_URL = '/overview/ethernetstatus'


class Ethernet(object):
    """ Ethernet device

    Args:
        session (verisure.session): Current session
    """

    def __init__(self, session):
        self._session = session

    def get(self):
        """ Get device overview """
        status = self._session.get(OVERVIEW_URL)
        return [Overview('ethernet', val) for val in status]



"""
Alarm device
"""
import time

from .overview import Overview

OVERVIEW_URL = '/remotecontrol'
COMMAND_URL = '/remotecontrol/armstatechange.cmd'
CHECK_STATE = '/remotecontrol/checkstate.cmd'


class Alarm(object):
    """ Alarm device

    Args:
        session (verisure.session): current session
    """

    def __init__(self, session):
        self._session = session

    def get(self):
        """ get device overview """
        status = self._session.get(OVERVIEW_URL)
        return [Overview('alarm', val) for val in status
                if val['type'] == 'ARM_STATE']

    def set(self, code, state):
        """ set status of alarm component

            Args:
                code (str): Personal alarm code (four digits)
                state (str): 'ARMED_HOME', 'ARMED_AWAY' or 'DISARMED'

        """
        data = {
            'code': code,
            'state': state
            }
        return not self._session.post(COMMAND_URL, data)

    def wait_while_pending(self, max_request_count=100):
        """ Wait for pending alarm to finish

            Args:
                max_request_count (int): maximum number of post requests

            Returns: retries if success else -1

        """

        for counter in range(max_request_count):
            data = {'counter': counter}
            response = self._session.json_to_dict(
                self._session.post(CHECK_STATE, data))
            if 'hasResult' not in response:
                break
            if 'hasPending' in response and not response['hasPending']:
                return counter
            counter = counter + 1
            time.sleep(1)
        return -1



"""
Climate device
"""

from .overview import Overview

OVERVIEW_URL = '/overview/climatedevice'


class Climate(object):
    """ Climate device

    Args:
        session (verisure.session): Current session
    """
    def __init__(self, session):
        self._session = session

    def get(self):
        """ get device overview """
        status = self._session.get(OVERVIEW_URL)
        return [Overview('climate', val) for val in status]



"""
Lock device
"""

from .overview import Overview

OVERVIEW_URL = '/remotecontrol'
COMMAND_URL = '/remotecontrol/lockunlock.cmd'


class Lock(object):
    """ Lock device

    Args:
        session (verisure.session): Current session
    """

    def __init__(self, session):
        self._session = session

    def get(self):
        """ Get device overview """
        status = self._session.get(OVERVIEW_URL)
        return [Overview('lock', val) for val in status
                if val['type'] == 'DOOR_LOCK']

    def set(self, code, device_id, state):
        """ set status of alarm component

            Args:
                code (str): Personal alarm code (four digits)
                device_id (str): lock device id
                state (str): 'LOCKED', or 'UNLOCKED'

        """
        data = {
            'code': code,
            'deviceLabel': device_id,
            'state': state
            }
        return not self._session.post(COMMAND_URL, data)
